# -*- coding: utf-8 -*-

from six import iteritems


def enum(**enums):
    """
    Create enums with custom values to help user set their params
    :param enums:
    :return:
    """
    reverse = dict((value, key) for key, value in iteritems(enums))
    enums['reverse_mapping'] = reverse
    return type('Enum', (), enums)


def auto_enum(*sequential, **named):
    """
    Create enum
    :param sequential:
    :param named:
    :return:
    """
    enums = dict(zip(sequential, range(1, len(sequential) + 1)), **named)
    return enum(**enums)


TYPE_OF_MATRIX = enum(R4M_PROPRIETARY_ROUTING=1,
                      R4M_TRAFFIC_ENGINE=3,
                      TRUCKING=6)

DIRECTIONS_METHOD = enum(R4M_PROPRIETARY_INTERNAL_NAVIGATION_SYSTEM=1,
                         TRUCKING=3)

ALGORITHM_TYPE = enum(TSP=1,
                      VRP=2,
                      CVRP_TW_SD=3,
                      CVRP_TW_MD=4,
                      TSP_TW=5,
                      TSP_TW_CR=6,
                      BBCVRP=7,
                      ALG_LEGACY_DISTRIBUTED=101,
                      ALG_NONE=100)

TRAVEL_MODE = enum(DRIVING='Driving',
                   WALKING='Walking',
                   TRUCKING='Trucking')

DISTANCE_UNIT = enum(MI='mi',
                     KM='km')

AVOID = enum(HIGHWAYS='Highways',
             TOLLS='Tolls',
             MINIMIZE_HIGHWAYS='minimizeHighways',
             MINIMIZE_TOLLS='minimizeTolls',
             NONE='')

OPTIMIZE = enum(DISTANCE='Distance',
                TIME='Time',
                TIME_WITH_TRAFFIC='timeWithTraffic')

METRIC = auto_enum('ROUTE4ME_METRIC_EUCLIDEAN',
                   'ROUTE4ME_METRIC_MANHATTAN',
                   'ROUTE4ME_METRIC_GEODESIC',
                   'ROUTE4ME_METRIC_MATRIX',
                   'ROUTE4ME_METRIC_EXACT_2D', )

DEVICE_TYPE = enum(WEB="web",
                   IPHONE="iphone",
                   IPAD="ipad",
                   ANDROID_PHONE="android_phone",
                   ANDROID_TABLET="android_tablet")

FORMAT = enum(CSV='csv',
              SERIALIZED='serialized',
              XML='xml',
              JSON='json')

OPTIMIZATION_STATE = auto_enum('OPTIMIZATION_STATE_INITIAL',
                               'OPTIMIZATION_STATE_MATRIX_PROCESSING',
                               'OPTIMIZATION_STATE_OPTIMIZING',
                               'OPTIMIZATION_STATE_OPTIMIZED',
                               'OPTIMIZATION_STATE_ERROR',
                               'OPTIMIZATION_STATE_COMPUTING_DIRECTIONS', )

ROUTE_PATH_OUTPUT = enum(NONE='None',
                         POINTS='Points')

UTURN = auto_enum('UTURN_DEPART_SHORTEST',
                  'UTURN_DEPART_TO_RIGHT')

LEFT_TURN = auto_enum('LEFTTURN_ALLOW',
                      'LEFTTURN_FORBID',
                      'LEFTTURN_MULTIAPPROACH')

TRUCK_HAZARDOUS_GOODS = enum(NONE='',
                             EXPLOSIVE='explosive',
                             GAS='gas',
                             FLAMMABLE='flammable',
                             COMBUSTIBLE='combustible',
                             ORGANIC='organic',
                             POISON='poison',
                             RADIOACTIVE='radioActive',
                             CORROSIVE='corrosive',
                             POISONOUSINHALATION='poisonousInhalation',
                             HARMFULTOWATER='harmfulToWater',
                             OTHER='other',
                             ALLHAZARDOUSGOODS='allHazardousGoods')

TERRITORY_TYPE = enum(CIRCLE='circle',
                      POLY='poly',
                      RECT='rect', )



# -*- coding: utf-8 -*-

import json

from .api_endpoints import ADDRESS_HOST, API_HOST
from .base import Base
from .exceptions import ParamValueException


class Optimization(Base):
    """
        An Optimization Problem is a collection of addresses that need to be
        visited. This is distinct from a Route, which is a sequence of
        addresses that need to be visited by a single vehicle and a single
        driver in a fixed time period. Solving an Optimization Problem
        results in a number of routes.
    """

    def __init__(self, api):
        """
        Optimization Instance
        :param api:
        :return:
        """
        Base.__init__(self, api)

    def get_optimizations(self, **kwargs):
        """
        Get optimizations using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['limit', 'offset', ]):
            self.response = self.api._request_get(API_HOST,
                                                  kwargs)
            response = self.response.json()
            return response
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_optimization(self, **kwargs):
        """
        Get optimization using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['optimization_problem_id', ]):
            self.response = self.api._request_get(API_HOST,
                                                  kwargs)
            response = self.response.json()
            return response
        else:
            raise ParamValueException('params', 'Params are not complete')

    def update_optimization(self, **kwargs):
        """
        Update optimization using PUT request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['optimization_problem_id',
                                               'addresses',
                                               'reoptimize']):
            self.response = self.api._request_put(API_HOST,
                                                  kwargs)
            response = self.response.json()
            return response
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_optimization(self, **kwargs):
        """
        Delete optimization using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        self.json_data = kwargs
        if self.check_required_params(kwargs, ['optimization_problem_ids', ]):
            data = json.dumps(self.json_data, ensure_ascii=False)
            self.response = self.api._request_delete(API_HOST,
                                                     self.params,
                                                     data=data)
            response = self.response.json()
            return response
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_address_from_optimization(self, **kwargs):
        """
        Delete Address from an optimization using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['optimization_problem_id',
                                               'route_destination_id']):
            self.response = self.api._request_delete(ADDRESS_HOST,
                                                     kwargs)
            response = self.response.json()
            return response
        else:
            raise ParamValueException('params', 'Params are not complete')



# -*- coding: utf-8 -*-

# ==============================================================================
# AUTOGENERATED
# python.sdk.version - AUTOGENERATED
# VERSION.py - MAINTAINER's. Don't edit, if you don't know what are you doing
# ==============================================================================

VERSION = (0, 0, 7, 0)
RELEASE_SUFFIX = ''

VERSION_STRING = '.'.join([str(x) for x in VERSION])
RELEASE_STRING = "v{}{}".format(VERSION_STRING, RELEASE_SUFFIX)

PROJECT = 'Route4Me Python SDK'
COPYRIGHT = '2016-2020 © Route4Me Python Team'
AUTHOR = 'Route4Me Python Team (SDK)'
AUTHOR_EMAIL = 'juan@route4me.com'
TITLE = 'route4me'
LICENSE = 'ISC'
BUILD = None   # TRAVIS_COMMIT
COMMIT = None  # TRAVIS_BUILD_NUMBER



# -*- coding: utf-8 -*-
# codebeat:disable[SIMILARITY, BLOCK_NESTING]

import json

from .api_endpoints import TERRITORY_HOST
from .base import Base
from .exceptions import ParamValueException


class Territory(Base):
    """
    Territory Management
    """

    def __init__(self, api, addresses=[]):
        """
        Territory Instance
        :param api:
        :return:
        """
        self.json_data = {}
        Base.__init__(self, api)

    def get_territories(self):
        """
        Get territories using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(self.params, ['api_key', ]):
            self.response = self.api._request_get(TERRITORY_HOST,
                                                  self.params)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_territory(self, **kwargs):
        """
        Get Territory using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['api_key', 'territory_id']):
            self.response = self.api._request_get(TERRITORY_HOST,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def add_territory(self, **kwargs):
        """
        Add territory using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(kwargs, ['territory_name',
                                               'territory_color',
                                               'territory']):
            self.response = self.api._request_post(TERRITORY_HOST,
                                                   self.params,
                                                   data=json.dumps(kwargs))
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_territory(self, **kwargs):
        """
        Delete territory using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['territory_id']):
            self.response = self.api._request_delete(TERRITORY_HOST,
                                                     kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def update_territory(self, territory_id, **kwargs):
        """
        Delete territory using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        self.params.update({'territory_id': territory_id})
        if self.check_required_params(kwargs, ['territory_name',
                                               'territory_color',
                                               'territory']):
            self.response = self.api._request_put(TERRITORY_HOST,
                                                  self.params,
                                                  data=json.dumps(kwargs))
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

# codebeat:enable[SIMILARITY, BLOCK_NESTING]



# -*- coding: utf-8 -*-

import json

from .api_endpoints import ORDERS_HOST
from .base import Base
from .exceptions import ParamValueException


class Order(Base):
    """
    Orders are transactional events.
    """

    REQUIRED_FIELDS = ('address_1', 'cached_lat', 'cached_lng',)

    def __init__(self, api):
        """
        Order Instance
        :param api:
        :return:
        """
        Base.__init__(self, api)

    def create_order(self, **kwargs):
        """
        Create an Order
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, self.REQUIRED_FIELDS):
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_post(ORDERS_HOST,
                                              self.params,
                                              data=data)
            try:
                return response.json()
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def get_order(self, **kwargs):
        """
        Get an Order
        :param kwargs:
        :return: API response content
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['order_id', 'api_key', ]):
            response = self.api._request_get(ORDERS_HOST,
                                             kwargs)
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def update_order(self, **kwargs):
        """
        Update an Order
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, self.REQUIRED_FIELDS):
            response = self.api._request_put(ORDERS_HOST,
                                             self.params,
                                             data=json.dumps(kwargs))
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def delete_order(self, **kwargs):
        """
        Delete an Order
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, ['order_ids', ]):
            response = self.api._request_delete(ORDERS_HOST,
                                                self.params,
                                                data=json.dumps(kwargs))
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')



# -*- coding: utf-8 -*-

import json

from .api_endpoints import (
    MEMBER_AUTHENTICATE,
    USER_LICENSE_HOST,
    VALIDATE_SESSION,
    WEBINAR_REGISTER,
    VERIFY_DEVICE_LICENSE,
    GET_USERS_HOST,
    USER_URL,
    REGISTER_ACTION
)
from .base import Base
from .exceptions import ParamValueException


class Members(Base):
    """
    Members management.
    """

    def __init__(self, api):
        """
        Members Instance
        :param api:
        :return:
        """
        Base.__init__(self, api)

    def app_purchase_user_license(self, **kwargs):
        """
        Application purchase user License
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, ['member_id',
                                               'session_guid',
                                               'device_id',
                                               'device_type',
                                               'subscription_name',
                                               'token',
                                               'payload',
                                               'format', ]):
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_post(USER_LICENSE_HOST,
                                              self.params,
                                              data=data)
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def verify_device_license(self, **kwargs):
        """
        Verify User License
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, ['device_id',
                                               'device_type',
                                               'format', ]):
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_post(VERIFY_DEVICE_LICENSE,
                                              self.params,
                                              data=data)
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def member_authenticate(self, **kwargs):
        """
        Member Authenticate
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, ['email',
                                               'password', ]):
            kwargs['strEmail'] = kwargs.pop('email')
            kwargs['strPassword'] = kwargs.pop('password')
            response = self.api._request_post(MEMBER_AUTHENTICATE,
                                              self.params,
                                              data=kwargs)
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def get_users(self, **kwargs):
        """
        Get users using GET request
        :return: API response
        """
        kwargs.update({'api_key': self.params['api_key'], })
        response = self.api._request_get(GET_USERS_HOST,
                                         kwargs)
        try:
            return response.json()
        except ValueError:
            return response.content

    def get_api_key_users(self, **kwargs):
        """
        Get users taht belong to a given api_key using GET request
        :return: API response
        """
        kwargs.update({'api_key': self.params['api_key'], })
        response = self.api._request_get(USER_URL,
                                         kwargs)
        try:
            return response.json()
        except ValueError:
            return response.content

    def validate_session(self, **kwargs):
        """
        Validate Session
        :param kwargs:
        :return: API response content
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['session_guid',
                                               'member_id', ]):
            response = self.api._request_get(VALIDATE_SESSION,
                                             kwargs)
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def webinar_registration(self, **kwargs):
        """
        Webinar Register
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, ["email_address",
                                               "first_name",
                                               "last_name",
                                               "phone_number",
                                               "company_name",
                                               "member_id",
                                               "webiinar_date"]):
            response = self.api._request_post(WEBINAR_REGISTER,
                                              self.params,
                                              data=kwargs)
            try:
                return json.loads(response.content)
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')

    def register(self, **kwargs):
        """
        Register Action
        :param kwargs:
        :return: API response content
        """
        if self.check_required_params(kwargs, ["industry",
                                               "first_name",
                                               "last_name",
                                               "email_address",
                                               "check_terms",
                                               "device_type",
                                               "plan",
                                               "password_1",
                                               "password_2",
                                               "format"]):
            params = {'plan': kwargs.pop('plan')}
            kwargs['strIndustry'] = kwargs.pop('industry')
            kwargs['strFirstName'] = kwargs.pop('first_name')
            kwargs['strLastName'] = kwargs.pop('last_name')
            kwargs['strEmail'] = kwargs.pop('email_address')
            kwargs['chkTerms'] = kwargs.pop('check_terms')
            kwargs['device_type'] = kwargs.pop('device_type')
            kwargs['strPassword_1'] = kwargs.pop('password_1')
            kwargs['strPassword_2'] = kwargs.pop('password_2')
            response = self.api._request_post(REGISTER_ACTION,
                                              params,
                                              data=kwargs)
            try:
                return response.json()
            except ValueError:
                return response.content
        else:
            raise ParamValueException('order', 'Missing required params')



# -*- coding: utf-8 -*-
# codebeat:disable[SIMILARITY, ABC]

import json

from .api_endpoints import AVOIDANCE
from .base import Base
from .exceptions import ParamValueException


class AvoindanceZones(Base):
    """
    Avoidance Zones Management
    """

    def __init__(self, api):
        """
        Avoidance Zones Instance
        :param api:
        :return:
        """
        self.json_data = {}
        Base.__init__(self, api)

    def get_avoidance_zones(self):
        """
        Get avoidance zones using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(self.params, ['api_key', ]):
            self.response = self.api._request_get(AVOIDANCE,
                                                  self.params)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_avoidance_zone(self, **kwargs):
        """
        Get avoidance zones using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['api_key', 'territory_id']):
            self.response = self.api._request_get(AVOIDANCE,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def add_avoidance_zone(self, **kwargs):
        """
        Add avoidance zone using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(kwargs, ['territory_name',
                                               'territory_color',
                                               'territory']):
            self.response = self.api._request_post(AVOIDANCE,
                                                   self.params,
                                                   data=json.dumps(kwargs))
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_avoidance_zone(self, **kwargs):
        """
        Delete avoidance zone using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['territory_id']):
            self.response = self.api._request_delete(AVOIDANCE,
                                                     kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def update_avoidance_zone(self, territory_id, **kwargs):
        """
        Delete avoidance zone using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        self.params.update({'territory_id': territory_id})
        if self.check_required_params(kwargs, ['territory_name',
                                               'territory_color',
                                               'territory']):
            self.response = self.api._request_put(AVOIDANCE,
                                                  self.params,
                                                  data=json.dumps(kwargs))
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

# codebeat:enable[SIMILARITY, ABC]



# -*- coding: utf-8 -*-
# codebeat:disable[TOO_MANY_FUNCTIONS, LOC, ABC, ARITY, TOTAL_LOC]

MAIN_HOST = 'https://api.route4me.com'

ACTIVITY_FEED = '{0}/api.v4/activity_feed.php'.format(MAIN_HOST)
ADDRESSBOOK = '{0}/api.v4/address_book.php'.format(MAIN_HOST)
ADDRESS_HOST = '{}/api.v4/address.php'.format(MAIN_HOST)
ADD_ROUTE_NOTES_HOST = '{0}/actions/addRouteNotes.php'.format(MAIN_HOST)
API_HOST = '{0}/api.v4/optimization_problem.php'.format(MAIN_HOST)
MEMBER_AUTHENTICATE = '{0}/actions/authenticate.php'.format(MAIN_HOST)
AVOIDANCE = '{0}/api.v4/avoidance.php'.format(MAIN_HOST)
BATCH_GEOCODER = '{0}/api/geocoder.php'.format(MAIN_HOST)
VERIFY_DEVICE_LICENSE = '{0}/api/device/verify_device_license.php'.format(MAIN_HOST)
DEVICE_LOCATION_URL = '{0}/api/track/get_device_location.php'.format(MAIN_HOST)
DUPLICATE_ROUTE = '{0}/actions/duplicate_route.php'.format(MAIN_HOST)
EXPORTER = '{0}/actions/route/export_current_route.php'.format(MAIN_HOST)
FILE_UPLOAD_HOST = '{0}/actions/upload/upload.php'.format(MAIN_HOST)
FILE_UPLOAD_PREVIEW_HOST = '{0}/actions/upload/csv-xls-preview.php'.format(MAIN_HOST)
FILE_UPLOAD_GEOCODE_HOST = '{0}/actions/upload/csv-xls-geocode.php'.format(MAIN_HOST)
GET_ACTIVITIES_HOST = '{0}/api/get_activities.php'.format(MAIN_HOST)
GET_USERS_HOST = '{0}/api/member/view_users.php'.format(MAIN_HOST)
MERGE_ROUTES_HOST = '{0}/actions/merge_routes.php'.format(MAIN_HOST)
MOVE_ROUTE_DESTINATION = '{0}/actions/route/move_route_destination.php'.format(MAIN_HOST)
ORDERS_HOST = '{0}/api.v4/order.php'.format(MAIN_HOST)
RAPID_ADDRESS = 'https://rapid.route4me.com/street_data/'
RAPID_ADDRESS_SERVICE = 'https://rapid.route4me.com/street_data/service/'
RAPID_ADDRESS_ZIP = 'https://rapid.route4me.com/street_data/zipcode/'
REGISTER_ACTION = '{0}/actions/register_action.php'.format(MAIN_HOST)
RESEQUENCE_ROUTE = '{0}/api.v3/route/reoptimize_2.php'.format(MAIN_HOST)
ROUTE_HOST = '{0}/api.v4/route.php'.format(MAIN_HOST)
SET_GPS_HOST = '{0}/track/set.php'.format(MAIN_HOST)
SHARE_ROUTE_HOST = '{0}/actions/route/share_route.php'.format(MAIN_HOST)
SHOW_ROUTE_HOST = '{0}/route4me.php'.format(MAIN_HOST)
SINGLE_GEOCODER = '{0}/api/address.php'.format(MAIN_HOST)
TERRITORY_HOST = '{0}/api.v4/territory.php'.format(MAIN_HOST)
USER_URL = '{0}/api.v4/user.php'.format(MAIN_HOST)
USER_LICENSE_HOST = '{0}/api/member/user_license.php'.format(MAIN_HOST)
VALIDATE_SESSION = '{0}/datafeed/session/validate_session.php'.format(MAIN_HOST)
VEHICLES_HOST = '{}/api/vehicles/view_vehicles.php'.format(MAIN_HOST)
WEBINAR_REGISTER = '{0}/actions/webinar_register.php'.format(MAIN_HOST)
ZIPCODE_TERRITORIES = 'https://rapid.route4me.com/street_data/territories'



# -*- coding: utf-8 -*-
# codebeat:disable[SIMILARITY, BLOCK_NESTING]

from .api_endpoints import VEHICLES_HOST
from .base import Base
from .exceptions import ParamValueException, APIException


class Vehicle(Base):
    """
    Vehicles Management
    """

    def __init__(self, api):
        """
        Vehicle Instance
        :param api:
        :return:
        """
        self.json_data = {}
        Base.__init__(self, api)

    def get_vehicles(self):
        """
        Get vehicles using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(self.params, ['api_key', ]):
            self.response = self.api._request_get(VEHICLES_HOST,
                                                  self.params)
            try:
                self.json_data = self.response.json()
                return self.json_data
            except ValueError:
                raise APIException(self.response.status_code,
                                   self.response.content,
                                   self.response.url)
        else:
            raise ParamValueException('params', 'Params are not complete')

# codebeat:enable[SIMILARITY, BLOCK_NESTING]



# -*- coding: utf-8 -*-

import json

from .api_endpoints import ACTIVITY_FEED
from .base import Base
from .exceptions import ParamValueException


class ActivityFeed(Base):
    """
    Activity Feed Management
    """

    def __init__(self, api):
        """
        Activity Feed Instance
        :param api:
        :return:
        """
        self.json_data = {}
        Base.__init__(self, api)

    def get_activities_feed(self, **kwargs):
        """
        Get Activity Feed using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['api_key', ]):
            self.response = self.api._request_get(ACTIVITY_FEED,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_activities_feed_by_type(self, **kwargs):
        """
        Get Activity Feed by Type using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['api_key', 'activity_type']):
            self.response = self.api._request_get(ACTIVITY_FEED,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_activity_feed_inserted(self, **kwargs):
        """
        Get Activity Feed Inserted using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'],
                       'activity_type': 'insert-destination', })
        if self.check_required_params(kwargs, ['api_key', 'route_id']):
            self.response = self.api._request_get(ACTIVITY_FEED,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_activity_feed_deleted(self, **kwargs):
        """
        Get Activity Feed Deleted using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'],
                       'activity_type': 'delete-destination', })
        if self.check_required_params(kwargs, ['api_key', 'route_id']):
            self.response = self.api._request_get(ACTIVITY_FEED,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_activity_feed_route_owner_changed(self, **kwargs):
        """
        Get Activity Feed Route Owner Changed using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'],
                       'activity_type': 'route-owner-changed', })
        if self.check_required_params(kwargs, ['api_key', 'route_id']):
            self.response = self.api._request_get(ACTIVITY_FEED,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def log_specific_message(self, **kwargs):
        """
        Activity Feed Log an Specific Message using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        self.json_data = kwargs
        kwargs.update({'api_key': self.params['api_key'],
                       'activity_type': 'user_message', })
        if self.check_required_params(self.json_data, ['api_key',
                                                       'activity_message',
                                                       'route_id']):
            data = json.dumps(self.json_data, ensure_ascii=False)
            self.response = self.api._request_post(ACTIVITY_FEED,
                                                   self.params, data=data)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')



# -*- coding: utf-8 -*-

import json

from .api_endpoints import ADDRESSBOOK
from .base import Base
from .exceptions import ParamValueException


class AddressBook(Base):
    """
    Address Book Management
    """
    REQUIRED_FIELDS = ('address_1', 'cached_lat', 'cached_lng',)

    def __init__(self, api, addresses=[]):
        """
        AddressBook Instance
        :param api:
        :return:
        """
        self.json_data = {}
        Base.__init__(self, api)

    def create_contact(self, **kwargs):
        """
        Create a contact in AddressBook using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        self.json_data = kwargs
        if self.check_required_params(self.json_data, self.REQUIRED_FIELDS):
            self.response = self.api._request_post(ADDRESSBOOK,
                                                   self.params,
                                                   json=self.json_data)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_addressbook_contacts(self, **kwargs):
        """
        Get contacts from AddressBook using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['api_key', ]):
            self.response = self.api._request_get(ADDRESSBOOK,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_addressbook_contact(self, **kwargs):
        """
        Get a contact from AddressBook using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['address_id', ]):
            self.response = self.api._request_get(ADDRESSBOOK,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def update_contact(self, **kwargs):
        """
        Update a contact from AddressBook using PUT request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(kwargs, ['address_id', ]):
            self.response = self.api._request_put(ADDRESSBOOK,
                                                  self.params,
                                                  json=kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_addressbook_contact(self, **kwargs):
        """
        Delete a contact from AddressBook using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(kwargs, ['address_ids', ]):
            self.response = self.api._request_delete(ADDRESSBOOK,
                                                     self.params,
                                                     data=json.dumps(kwargs))
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')



# -*- coding: utf-8 -*-
# codebeat:disable[TOTAL_LOC, TOO_MANY_FUNCTIONS, TOTAL_COMPLEXITY]

import re

from .constants import (
    TRAVEL_MODE,
    OPTIMIZE, DEVICE_TYPE,
    DISTANCE_UNIT,
    ROUTE_PATH_OUTPUT,
    TRUCK_HAZARDOUS_GOODS,
    FORMAT,
)
from .exceptions import ParamValueException


class Base(object):
    """
    Base Class, with common methods.
    """

    def __init__(self, api):
        """
        Base instance
        :param api:
        :return:
        """
        self.response = None
        self.api = api
        self.data = {'parameters': {},
                     'addresses': {},
                     }
        self.params = {'api_key': api.key, }

    @staticmethod
    def check_string_type(obj):
        """
        Check if an object is string type
        :param obj:
        :return:
        """
        try:
            return isinstance(obj, basestring)
        except NameError:
            return isinstance(obj, str)

    def format(self, set_format):
        """
        Add format to params.
        :param set_format:
        :return:
        :raise: ParamValueException if set_format is not in FORMAT
        """
        if set_format in FORMAT.reverse_mapping.keys():
            self._copy_param({'format': set_format})
        else:
            raise ParamValueException('format', 'Must be CSV, SERIALIZED, XML')

    def member_id(self, member_id, target='data'):
        """
        Set member_id in params or data
        :param member_id:
        :param target:
        :return:
        :raise: ParamValueException if member_id is not Integer
        """
        if isinstance(member_id, int):
            getattr(self, '_copy_%s' % target)({'member_id': member_id})
        else:
            raise ParamValueException('member_id', 'Must be integer')

    def route_id(self, route_id):
        """
        Set route_id in params or data
        :param route_id:
        :return:
        :raise: ParamValueException if route_id is not String
        """
        if self.check_string_type(route_id):
            self._copy_param({'route_id': route_id})
        else:
            raise ParamValueException('route_id', 'Must be String')

    def address_1(self, address_1):
        """
        Set address_1 in params or data
        :param address_1:
        :return:
        :raise: ParamValueException if address_1 is not String
        """
        if self.check_string_type(address_1):
            self._copy_param({'address_1': address_1})
        else:
            raise ParamValueException('address_1', 'Must be String')

    def tx_id(self, tx_id):
        """
        Set tx_id param
        :param tx_id:
        :return:
        """
        if self.check_string_type(tx_id):
            self._copy_param({'tx_id': tx_id})
        else:
            raise ParamValueException('tx_id', 'Must be String')

    def vehicle_id(self, vehicle_id):
        """
        Set vehicle_id param
        :param vehicle_id:
        :return:
        """
        if isinstance(vehicle_id, int):
            self._copy_data({'vehicle_id': vehicle_id})
        else:
            raise ParamValueException('vehicle_id', 'Must be integer')

    def course(self, course):
        """
        Set course param
        :param course:
        :return:
        """
        if isinstance(course, int):
            self._copy_param({'course': course})
        else:
            raise ParamValueException('course', 'Must be integer')

    def speed(self, speed):
        """
        Set speed param
        :param speed:
        :return:
        """
        if isinstance(speed, int):
            self._copy_param({'speed': speed})
        else:
            raise ParamValueException('speed', 'Must be Float')

    def lat(self, lat):
        """
        Set lat param
        :param lat:
        :return:
        """
        if isinstance(lat, float):
            self._copy_param({'lat': lat})
        else:
            raise ParamValueException('lat', 'Must be Float')

    def lng(self, lng):
        """
        Set lng param
        :param lng:
        :return:
        """
        if isinstance(lng, float):
            self._copy_param({'lng': lng})
        else:
            raise ParamValueException('lng', 'Must be Float')

    def cached_lat(self, lat):
        """
        Set lat param
        :param lat:
        :return:
        """
        if isinstance(lat, float):
            self._copy_param({'lat': lat})
        else:
            raise ParamValueException('lat', 'Must be Float')

    def cached_lng(self, lng):
        """
        Set lng param
        :param lng:
        :return:
        """
        if isinstance(lng, float):
            self._copy_param({'lng': lng})
        else:
            raise ParamValueException('lng', 'Must be Float')

    def altitude(self, altitude):
        """
        Set altitude param
        :param altitude:
        :return:
        """
        if isinstance(altitude, float):
            self._copy_param({'altitude': altitude})
        else:
            raise ParamValueException('altitude', 'Must be Float')

    def device_guid(self, device_guid):
        """
        Set device_guid param
        :param device_guid:
        :return:
        """
        if self.check_string_type(device_guid):
            self._copy_param({'device_guid': device_guid})
        else:
            raise ParamValueException('device_guid', 'Must be String')

    def app_version(self, app_version):
        """
        Set app_version param
        :param app_version:
        :return:
        """
        if self.check_string_type(app_version):
            self._copy_param({'app_version': app_version})
        else:
            raise ParamValueException('app_version', 'Must be String')

    def device_timestamp(self, device_timestamp):
        """
        Set device_timestamp param. Must be a valid date time
        with this format:  YYYY-MM-DD HH:MM:SS
        :param device_timestamp:
        :return:
        """
        pattern = r'^(\d{4})-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])([01][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])$'
        if re.match(pattern, device_timestamp):
            self._copy_param({'device_timestamp': device_timestamp})
        else:
            raise ParamValueException('device_timestamp',
                                      'Must be YYYY-MM-DD HH:MM:SS format')

    def algorithm_type(self, algorithm_type):
        """
        Set algorithm_type. Choices are:
        TSP, VRP, CVRP_TW_SD
        CVRP_TW_MD, TSP_TW,
        TSP_TW_CR and BBCVRP
        :param: algorithm_type:
        :return:
        """
        VALID = [1, 2, 3, 4, 5, 6, 7, 100, 101, ]
        if algorithm_type in VALID:
            self._copy_data({'algorithm_type': algorithm_type})
        else:
            raise ParamValueException('algorithm_type',
                                      'Must be ALGORITHM_TYPE: '
                                      'TSP(1), VRP(2), CVRP_TW_SD(3), '
                                      'CVRP_TW_MD(4), TSP_TW(5), '
                                      'TSP_TW_CR(6), BBCVRP(7), '
                                      'NO OPTIMIZATION(100) or '
                                      'LEGACY_DISTRIBUTED(101)')

    def route_name(self, route_name):
        """
        Set route_name param
        :param route_name:
        :return:
        """
        if self.check_string_type(route_name):
            self._copy_data({'route_name': route_name})
        else:
            raise ParamValueException('route_name', 'Must be String')

    def optimization_problem_id(self, optimization_problem_id):
        """
        Set optimization_problem_id param
        :param optimization_problem_id:
        :return:
        """
        if self.check_string_type(optimization_problem_id):
            self._copy_param({'optimization_problem_id':
                              optimization_problem_id})
        else:
            raise ParamValueException('optimization_problem_id',
                                      'Must be String')

    def optimized_callback_url(self, optimized_callback_url):
        """
        Set optimized_callback_url param
        :param optimized_callback_url:
        :return:
        """
        if self.check_string_type(optimized_callback_url):
            self._copy_param({'optimized_callback_url': optimized_callback_url})
        else:
            raise ParamValueException('optimized_callback_url',
                                      'Must be String')

    def remote_ip(self, remote_ip):
        """
        Set remote_ip param
        :param remote_ip:
        :return:
        """
        if isinstance(remote_ip, int):
            self._copy_data({'remote_ip': remote_ip})
        else:
            raise ParamValueException('remote_ip', 'Must be integer')

    def travel_mode(self, travel_mode):
        """
        Set travel_mode. Options are:
        DRIVING, WALKING, TRUCKING
        :param travel_mode:
        :return:
        """
        if travel_mode in TRAVEL_MODE.reverse_mapping.keys():
            self._copy_data({'travel_mode': travel_mode})
        else:
            raise ParamValueException('travel_mode', 'Must be DRIVING, '
                                                     'WALKING, TRUCKING')

    def optimize(self, optimize):
        """
        Set optimize param
        :param optimize:
        :return:
        """
        if optimize in OPTIMIZE.reverse_mapping.keys():
            self._copy_data({'optimize': optimize})
        else:
            raise ParamValueException('optimize', 'Must be DISTANCE, TIME, '
                                                  'TIME_WITH_TRAFFIC')

    def distance_unit(self, distance_unit):
        """
        Set distance_unit param
        :param distance_unit:
        :return:
        """
        if distance_unit in DISTANCE_UNIT.reverse_mapping.keys():
            self._copy_data({'distance_unit': distance_unit})
        else:
            raise ParamValueException('distance_unit', 'Must be MI or KM')

    def device_type(self, device_type, target='data'):
        """
        Set device_type. Options are: WEB, IPHONE, IPAD, ANDROID_PHONE,
        ANDROID_TABLET
        :param device_type:
        :param target:
        :return:
        """
        if device_type in DEVICE_TYPE.reverse_mapping.keys():
            getattr(self, '_copy_%s' % target)({'device_type': device_type})
        else:
            raise ParamValueException('device_type', 'Must be WEB, IPHONE, '
                                                     'IPAD, ANDROID_PHONE, '
                                                     'ANDROID_TABLET')

    def route_path_output(self, route_path_output):
        """
        Set device_type. Options are: WEB, IPHONE, IPAD, ANDROID_PHONE,
        ANDROID_TABLET
        :param route_path_output:
        :param target:
        :return:
        """
        if route_path_output in ROUTE_PATH_OUTPUT.reverse_mapping.keys():
            self._copy_param({'route_path_output': route_path_output})
        else:
            raise ParamValueException('route_path_output', 'Must be NONE or '
                                                           'POINTS')

    def route_time(self, route_time):
        """
        Set route_time param
        :param route_time:
        :return:
        """
        if isinstance(route_time, int):
            self._copy_data({'route_time': route_time})
        else:
            raise ParamValueException('route_time', 'Must be integer')

    def trailer_weight_t(self, trailer_weight_t):
        """
        Set trailer_weight_t param
        :param trailer_weight_t:
        :return:
        """
        if isinstance(trailer_weight_t, int):
            self._copy_data({'trailer_weight_t': trailer_weight_t})
        else:
            raise ParamValueException('trailer_weight_t',
                                      'Must be integer')

    def limited_weight_t(self, limited_weight_t):
        """
        Set limited_weight_t param
        :param limited_weight_t:
        :return:
        """
        if isinstance(limited_weight_t, float) or \
           isinstance(limited_weight_t, int):
            self._copy_data({'limited_weight_t': limited_weight_t})
        else:
            raise ParamValueException('limited_weight_t',
                                      'Must be integer')

    def weight_per_axle_t(self, weight_per_axle_t):
        """
        Set weight_per_axle_t param
        :param weight_per_axle_t:
        :return:
        """
        if isinstance(weight_per_axle_t, float) or \
           isinstance(weight_per_axle_t, int):
            self._copy_data({'weight_per_axle_t': weight_per_axle_t})
        else:
            raise ParamValueException('weight_per_axle_t',
                                      'Must be integer')

    def truck_height(self, truck_height):
        """
        Set truck_height param
        :param truck_height:
        :return:
        """
        if isinstance(truck_height, float) or \
           isinstance(truck_height, int):
            self._copy_data({'truck_height': truck_height})
        else:
            raise ParamValueException('truck_height', 'Must be integer')

    def truck_width(self, truck_width):
        """
        Set truck_width param
        :param truck_width:
        :return:
        """
        if isinstance(truck_width, float) or \
           isinstance(truck_width, int):
            self._copy_data({'truck_width': truck_width})
        else:
            raise ParamValueException('truck_width', 'Must be integer')

    def truck_length(self, truck_length):
        """
        Set truck_length param
        :param truck_length:
        :return:
        """
        if isinstance(truck_length, float) or \
           isinstance(truck_length, int):
            self._copy_data({'truck_length': truck_length})
        else:
            raise ParamValueException('truck_length', 'Must be integer')

    def min_tour_size(self, min_tour_size):
        """
        Set min_tour_size param
        :param min_tour_size:
        :return:
        """
        if isinstance(min_tour_size, int):
            self._copy_data({'min_tour_size': min_tour_size})
        else:
            raise ParamValueException('min_tour_size',
                                      'Must be integer')

    def route_date(self, route_date):
        """
        Set route_date param
        :param route_date:
        :return:
        """
        if isinstance(route_date, int):
            self._copy_data({'route_date': route_date})
        else:
            raise ParamValueException('route_date', 'Must be integer')

    def route_max_duration(self, route_max_duration):
        """
        Set route_max_duration param
        :param route_max_duration:
        :return:
        """
        if isinstance(route_max_duration, int):
            self._copy_data({'route_max_duration': route_max_duration})
        else:
            raise ParamValueException('route_max_duration',
                                      'Must be integer')

    def vehicle_capacity(self, vehicle_capacity):
        """
        Set vehicle_capacity param
        :param vehicle_capacity:
        :return:
        """
        if isinstance(vehicle_capacity, int):
            self._copy_data({'vehicle_capacity': vehicle_capacity})
        else:
            raise ParamValueException('vehicle_capacity',
                                      'Must be integer')

    def parts(self, parts):
        """
        Set parts param
        :param parts:
        :return:
        """
        if isinstance(parts, int):
            self._copy_data({'parts': parts})
        else:
            raise ParamValueException('parts',
                                      'Must be integer')

    def limit(self, limit):
        """
        Set limit param
        :param limit:
        :return:
        """
        if isinstance(limit, int):
            self._copy_param({'limit': limit})
        else:
            raise ParamValueException('limit',
                                      'Must be integer')

    def offset(self, offset):
        """
        Set offset param
        :param offset:
        :return:
        """
        if isinstance(offset, int):
            self._copy_param({'offset': offset})
        else:
            raise ParamValueException('offset',
                                      'Must be integer')

    def vehicle_max_distance_mi(self, vehicle_max_distance_mi):
        """
        Set vehicle_max_distance_mi
        :param vehicle_max_distance_mi:
        :return:
        """
        if isinstance(vehicle_max_distance_mi, int):
            self._copy_data({'vehicle_max_distance_mi':
                             vehicle_max_distance_mi})
        else:
            raise ParamValueException('vehicle_max_distance_mi',
                                      'Must be integer')

    def max_tour_size(self, max_tour_size):
        """
        Set max_tour_size
        :param max_tour_size:
        :return:
        """
        if isinstance(max_tour_size, int):
            self._copy_data({'max_tour_size':
                             max_tour_size})
        else:
            raise ParamValueException('max_tour_size',
                                      'Must be integer')

    def route_email(self, route_email):
        """
        Set route_email param
        :param route_email:
        :return:
        """
        if self.check_string_type(route_email):
            self._copy_data({'route_email': route_email})
        else:
            raise ParamValueException('route_email', 'Must be String')

    def metric(self, metric):
        """
        Set metric Param. Must be:
        ROUTE4ME_METRIC_EUCLIDEA
        ROUTE4ME_METRIC_MANHATTA
        ROUTE4ME_METRIC_GEODESIC
        ROUTE4ME_METRIC_MATRIX
        ROUTE4ME_METRIC_EXACT_2D
        :param metric:
        :return:
        """
        if 1 <= metric <= 7:
            self._copy_data({'metric': metric})
        else:
            raise ParamValueException('metric',
                                      'Must be METRIC: '
                                      'ROUTE4ME_METRIC_EUCLIDEAN , '
                                      'ROUTE4ME_METRIC_MANHATTAN, '
                                      'ROUTE4ME_METRIC_GEODESIC'
                                      'ROUTE4ME_METRIC_MATRIX'
                                      'ROUTE4ME_METRIC_EXACT_2D')

    def store_route(self, store_route):
        """
        Set store_route param
        :param store_route:
        :return:
        """
        if 0 <= store_route <= 1:
            self._copy_data({'store_route': store_route})
        else:
            raise ParamValueException('store_route', 'Must be 0 or 1')

    def lock_last(self, lock_last):
        """
        Set lock_last param
        :param lock_last:
        :return:
        """
        if 0 <= lock_last <= 1:
            self._copy_data({'lock_last': lock_last})
        else:
            raise ParamValueException('lock_last', 'Must be 0 or 1')

    def disable_optimization(self, disable_optimization):
        """
        Set disable_optimization param
        :param disable_optimization:
        :return:
        """
        if 0 <= disable_optimization <= 1:
            self._copy_data({'disable_optimization': disable_optimization})
        else:
            raise ParamValueException('disable_optimization', 'Must be 0 or 1')

    def shared_publicly(self, shared_publicly):
        """
        Set shared_publicly param
        :param shared_publicly:
        :return:
        """
        if 0 <= shared_publicly <= 1:
            self._copy_data({'shared_publicly': shared_publicly})
        else:
            raise ParamValueException('shared_publicly', 'Must be 0 or 1')

    def reoptimize(self, reoptimize):
        """
        Set reoptimize param
        :param reoptimize:
        :return:
        """
        if 0 <= reoptimize <= 1:
            self._copy_param({'reoptimize': reoptimize})
        else:
            raise ParamValueException('reoptimize', 'Must be 0 or 1')

    def share_route(self, share_route):
        """
        Set share_route param
        :param share_route:
        :return:
        """
        if 0 <= share_route <= 1:
            self._copy_data({'share_route': share_route})
        else:
            raise ParamValueException('share_route', 'Must be 0 or 1')

    def rt(self, rt):
        """
        Set rt param.
        :param rt:
        :return:
        """
        if 0 <= rt <= 1:
            self._copy_data({'rt': rt})
        else:
            raise ParamValueException('rt', 'Must be 0 or 1')

    def has_trailer(self, has_trailer):
        """
        Set has_trailer param.
        :param has_trailer:
        :return:
        """
        if 0 <= has_trailer <= 1:
            self._copy_data({'has_trailer': has_trailer})
        else:
            raise ParamValueException('has_trailer', 'Must be 0 or 1')

    def optimization_quality(self, optimization_quality):
        """
        Set optimization_quality param.
        :param optimization_quality:
        :return:
        """
        if 1 <= optimization_quality <= 3:
            self._copy_data({'optimization_quality': optimization_quality})
        else:
            raise ParamValueException('optimization_quality',
                                      'Must be between 1 to 3')

    def directions(self, directions):
        """
        Set directions param.
        :param directions:
        :return:
        """
        if 0 <= directions <= 1:
            self._copy_param({'directions': directions})
        else:
            raise ParamValueException('directions', 'Must be 0 or 1')

    def device_tracking_history(self, device_tracking_history):
        """
        Set device_tracking_history param.
        :param device_tracking_history:
        :return:
        """
        if 0 <= device_tracking_history <= 1:
            self._copy_param({
                'device_tracking_history': device_tracking_history
            })
        else:
            raise ParamValueException('device_tracking_history',
                                      'Must be 0 or 1')

    def uturn(self, uturn_type):
        """
        Set uturn_type. Choices are:
        UTURN_DEPART_SHORTEST or
        UTURN_DEPART_TO_RIGHT
        :param: uturn_type:
        :return:
        """
        if 1 <= uturn_type <= 2:
            self._copy_data({'uturn': uturn_type})
        else:
            raise ParamValueException('uturn',
                                      'Must be : '
                                      'UTURN_DEPART_SHORTEST or '
                                      'UTURN_DEPART_TO_RIGHT')

    def leftturn(self, left_turn_type):
        """
        Set leftturn. Choices are:
        LEFTTURN_ALLOW or
        LEFTTURN_FORBID or
        LEFTTURN_MULTIAPPROACH
        :param: leftturn:
        :return:
        """
        if 1 <= left_turn_type <= 3:
            self._copy_data({'leftturn': left_turn_type})
        else:
            raise ParamValueException('leftturn',
                                      'Must be : '
                                      'LEFTTURN_ALLOW or '
                                      'LEFTTURN_FORBID or '
                                      'LEFTTURN_MULTIAPPROACH')

    def dm(self, dm):
        """
        Set dm. Choices are:
        R4M_PROPRIETARY_ROUTING or
        R4M_TRAFFIC_ENGINE or
        TRUCKING
        :param: dm:
        :return:
        """
        if dm in [1, 3, 6]:
            self._copy_data({'dm': dm})
        else:
            raise ParamValueException(
                'dm',
                'Must be : '
                'R4M_PROPRIETARY_ROUTING or '
                'R4M_TRAFFIC_ENGINE or '
                'TRUCKING'
            )

    def dirm(self, dirm):
        """
        Set dirm. Choices are:
        R4M_PROPRIETARY_INTERNAL_NAVIGATION_SYSTEM or
        TRUCKING
        :param: dirm:
        :return:
        """
        if dirm in [1, 3]:
            self._copy_data({'dirm': dirm})
        else:
            raise ParamValueException(
                'dirm',
                'Must be : '
                'R4M_PROPRIETARY_INTERNAL_NAVIGATION_SYSTEM or'
                'TRUCKING'
            )

    def _copy_data(self, params):
        """
        Copy params to data
        :param params:
        :return:
        """
        self.data['parameters'].update(params)

    def _copy_param(self, params):
        """
        Copy params to params
        :param params:
        :return:
        """
        self.params.update(params)

    def get_params(self):
        """
        Get params
        :return:  params
        """
        return self.params

    def required_params(self, requirements=[]):
        """
        Check if required params are set
        :param requirements:
        :return:
        """
        return set(requirements).issubset(set(self.params.keys()))

    @staticmethod
    def check_required_params(params, requirements=[]):
        """
        Check if required params are set
        :param requirements:
        :return:
        """
        return set(requirements).issubset(set(params.keys()))

    def validate_params(self, **kwargs):
        """
        Validate params
        :param kwargs:
        :return:
        """
        for k, v in kwargs.items():
            try:
                self.__getattribute__(k)(v)
            except ParamValueException as e:
                raise e
            except AttributeError:
                raise ParamValueException(k, 'Not supported')
        return True

    def add(self, params={}, data={}):
        """
        Add params and data
        :param params:
        :param data:
        :return:
        """
        self.validate_params(**params)
        self.validate_params(**data)
        self.data.update(data)
        self.params.update(params)

    def truck_hazardous_goods(self, hazardous_goods):
        """
        Set truck_hazardous_goods param
        :param truck_hazardous_goods:
        :return:
        """
        if hazardous_goods in TRUCK_HAZARDOUS_GOODS.reverse_mapping.keys():
            self._copy_data({'truck_hazardous_goods': hazardous_goods})
        else:
            raise ParamValueException('truck_hazardous_goods',
                                      'Must be MI or KM')

# codebeat:enable[TOTAL_LOC, TOO_MANY_FUNCTIONS, TOTAL_COMPLEXITY]



# -*- coding: utf-8 -*-

from .api_endpoints import SET_GPS_HOST, DEVICE_LOCATION_URL
from .base import Base
from .exceptions import ParamValueException


class GPS(Base):
    """
        Set GPS position of the device
    """

    requirements = ('api_key',
                    'format',
                    'member_id',
                    'route_id',
                    'course',
                    'speed',
                    'lat',
                    'lng',
                    'device_type',
                    'device_guid',
                    'device_timestamp',
                    )

    def __init__(self, api):
        self.response = None
        self.params = {'api_key': api.key, }
        Base.__init__(self, api)

    def set_gps_track(self, **kwargs):
        """
        Set GPS position of device using GET request
        :return: Response status
        :raise: ParamValueException if any required param is not set
        """
        kwargs.update(self.params)
        if self.check_required_params(kwargs, self.requirements):
            self.response = self.api._request_get(SET_GPS_HOST,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_locations(self, **kwargs):
        """
        Get GPS tracks of a vehicle using GET request
        :return: Response status
        :raise: ParamValueException if any required param is not set
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, [
            'api_key',
            'format',
            'route_id',
            'member_id',
            'time_period',
        ]):
            self.response = self.api._request_get(DEVICE_LOCATION_URL,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')



# -*- coding: utf-8 -*-

# codebeat:disable[TOO_MANY_FUNCTIONS]

import json

from .api_endpoints import ROUTE_HOST, EXPORTER, \
    ADDRESS_HOST, GET_ACTIVITIES_HOST, DUPLICATE_ROUTE, SHARE_ROUTE_HOST, \
    MERGE_ROUTES_HOST, RESEQUENCE_ROUTE
from .base import Base
from .exceptions import ParamValueException


class Route(Base):
    """
    A Route is a multi-sequence of addresses that need to be
    visited by a single vehicle and a single driver in a fixed time period.
    """
    requirements = [
        'api_key',
        'route_id',
    ]

    def __init__(self, api):
        """
        Routes
        :param api: route4me instance
        :return: route instance
        """
        self.response = None
        self.params = {'api_key': api.key, }
        Base.__init__(self, api)

    def get_route(self, **kwargs):
        """
        Get route using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.validate_params(**kwargs):
            self.params.update(kwargs)
        if self.check_required_params(self.params, self.requirements):
            self.response = self.api._request_get(ROUTE_HOST,
                                                  self.params)
            return self.response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_route_tracking(self, **kwargs):
        """
        Get route using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.validate_params(**kwargs):
            self.params.update(kwargs)
        self.params.update({'device_tracking_history': 1})
        if self.check_required_params(self.params, self.requirements):
            self.response = self.api._request_get(ROUTE_HOST,
                                                  self.params)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_routes(self, **kwargs):
        """
        Get routes using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['limit', 'offset', ]):
            self.response = self.api._request_get(ROUTE_HOST,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_activities(self, **kwargs):
        """
        Get routes activities using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['route_id',
                                               'limit',
                                               'offset', ]):
            self.response = self.api._request_get(GET_ACTIVITIES_HOST,
                                                  kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def duplicate_route(self, **kwargs):
        """
        Duplicate route using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['route_id', ]):
            self.response = self.api._request_get(DUPLICATE_ROUTE,
                                                  kwargs)
            print(self.response.content)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_routes(self, **kwargs):
        """
        Delete routes using DELETE request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['route_id', ]):
            kwargs['route_id'] = ','.join(kwargs['route_id'])
            self.response = self.api._request_delete(ROUTE_HOST,
                                                     kwargs)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def delete_route(self, **kwargs):
        """
        Delete given route
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['route_id', ]):
            self.response = self.api._request_delete(ROUTE_HOST, kwargs)
            response = self.response.json()
            try:
                response = response.get('deleted')
                return response
            except AttributeError:
                return response.errors
        else:
            raise ParamValueException('params', 'Params are not complete')

    def insert_address_into_route(self, addresses, route_id):
        params = {'route_id': route_id}
        response = self._update_route(params, addresses)
        return response.json()

    def move_addresses_from_route(self, addresses, route_id):
        params = {'route_id': route_id}
        response = self._update_route(params, addresses)
        return response.json()

    def update_route_parameters(self, data, route_id):
        params = {'route_id': route_id}
        response = self._update_route(params, data)
        return response.json()

    def update_route(self, data, route_id):
        params = {'route_id': route_id}
        response = self._update_route(params, data)
        return response.json()

    def insert_address_into_route_optimal_position(self, **kwargs):
        """
        Insert an address into a route in an optimal position using PUT
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        if self.check_required_params(kwargs, ['addresses',
                                               'route_id',
                                               'optimal_position']):
            params = {'api_key': self.params['api_key'],
                      'route_id': kwargs.pop('route_id')}
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_put(ROUTE_HOST,
                                             params,
                                             data=data)
            return response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_route_path_points(self, **kwargs):
        """
        Get path point from a route using GET
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['route_path_output',
                                               'route_id', ]):
            response = self.api._request_get(ROUTE_HOST,
                                             kwargs)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_route_directions(self, **kwargs):
        """
        Get route directions using GET
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['directions', 'route_id', ]):
            response = self.api._request_get(ROUTE_HOST,
                                             kwargs)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def share_route(self, **kwargs):
        """
        Share a route using POST
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        if self.check_required_params(kwargs, ['recipient_email',
                                               'route_id',
                                               'response_format']):
            params = {'api_key': self.params['api_key'],
                      'route_id': kwargs.pop('route_id'),
                      'response_format': kwargs.pop('response_format')}
            response = self.api._request_post(SHARE_ROUTE_HOST,
                                              params,
                                              json=kwargs)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def resequence_route(self, **kwargs):
        """
        Resequence a route using PUT
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        if self.check_required_params(kwargs, ['addresses',
                                               'route_id',
                                               'route_destination_id']):
            params = {
                'api_key': self.params['api_key'],
                'route_id': kwargs.pop('route_id'),
                'route_destination_id': kwargs.pop('route_destination_id'),
            }
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_put(ROUTE_HOST,
                                             params, data=data)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def merge_routes(self, **kwargs):
        """
        Merge routes using POST
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        if self.check_required_params(kwargs, ['route_ids']):
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_post(MERGE_ROUTES_HOST,
                                              self.params,
                                              data=data)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def resequence_route_all(self, **kwargs):
        """
        Resequence Route using GET
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['disable_optimization',
                                               'route_id',
                                               'optimize', ]):
            response = self.api._request_get(RESEQUENCE_ROUTE,
                                             kwargs)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def update_route_destination_custom_data(self, **kwargs):
        """
        Update route destination custom data using PUT
        :return: API response
        :raises: ParamValueException if required params are not present.
        AttributeError if there is an error deleting a route
        """
        if self.check_required_params(kwargs, ['route_id',
                                               'route_destination_id']):
            params = {
                'api_key': self.params['api_key'],
                'route_id': kwargs.pop('route_id'),
                'route_destination_id': kwargs.pop('route_destination_id'),
            }
            data = json.dumps(kwargs, ensure_ascii=False)
            response = self.api._request_put(ADDRESS_HOST,
                                             params, data=data)
            return response.json()

        else:
            raise ParamValueException('params', 'Params are not complete')

    def export_route(self, route_id, output_format='csv'):
        """
        Get Route from given post data
        :param route_id:
        :param output_format:
        :return: response as a object
        """
        data = {'route_id': route_id, 'strExportFormat': output_format}
        self.response = self.api._make_request(EXPORTER, {}, data,
                                               self.api._request_post)
        return self.response.content

    def _update_route(self, params, data):
        params.update({'api_key': self.api.key})
        return self.api._request_put(ROUTE_HOST, request_params=params, json=data)

# codebeat:enable[TOO_MANY_FUNCTIONS]



# -*- coding: utf-8 -*-

DRIVER_VERSION = 'route4me-python-driver-0.0.1'


class APIException(Exception):
    """
    Handle API Exceptions
    """

    def __init__(self, status_code, response, url):
        self.status_code = status_code
        self.response = response
        self.url = url
        exception = {'http_status_code': status_code,
                     'response': response,
                     'url': url,
                     'driver_version': DRIVER_VERSION}
        Exception.__init__(self, exception)

    def get_status_code(self):
        """
        Status code
        :return:
        """
        return self.status_code

    def get_response(self):
        """
        Return response
        :return:
        """
        return self.response


class ParamValueException(Exception):
    """
    Handle Params exceptions
    """

    def __init__(self, param, msg):
        self.param = param
        self.msg = msg
        exception = {'param': param,
                     'msg': msg, }
        Exception.__init__(self, exception)

    def get_msg(self):
        """
        Message from exception
        :return:
        """
        return self.msg



# -*- coding: utf-8 -*-

import json
from collections import namedtuple


def _json_object_hook(d):
    """
    JSON to object helper
    :param d: data
    :return: namedtuple
    """
    keys = []
    for k in d.keys():
        if k[0].isdigit():
            k = 'd_{}'.format(k)
        keys.append(k)

    return namedtuple('X', keys)(*d.values())


def json2obj(data):
    """
    Parse JSON to object
    :param data: JSON data
    :return: object
    """
    return json.loads(data, object_hook=_json_object_hook)



# -*- coding: utf-8 -*-

from .api import Route4Me
from .exceptions import APIException, ParamValueException

from .version import PROJECT
from .version import COPYRIGHT
from .version import AUTHOR
from .version import TITLE
from .version import LICENSE
from .version import VERSION_STRING
from .version import RELEASE_STRING
from .version import BUILD
from .version import COMMIT

__project__ = PROJECT
__copyright__ = COPYRIGHT
__author__ = AUTHOR
__title__ = TITLE
__license__ = LICENSE
__version__ = VERSION_STRING
__release__ = RELEASE_STRING
__build__ = BUILD
__commit__ = COMMIT


__all__ = [
    'Route4Me',
    'APIException',
    'ParamValueException',
    '__project__',
    '__copyright__',
    '__author__',
    '__title__',
    '__license__',
    '__version__',
    '__release__',
    '__build__',
    '__commit__',

]



# -*- coding: utf-8 -*-
# codebeat:disable[ABC]

import json
import random
import time
import requests

from .api_endpoints import (
    ADD_ROUTE_NOTES_HOST,
    BATCH_GEOCODER,
    ADDRESS_HOST,
    SINGLE_GEOCODER,
)
from .base import Base
from .exceptions import ParamValueException


class Address(Base):
    """
    An Address is a destination in a route or optimization problem.
    Addresses can be depots, which means they are a departure points.
    Addresses can belong to only one route and one optimization problem,
    except for depots. One depot can be part of many routes if we have a
    VRP (multi-route) solution.
    """
    REQUIRED_FIELDS = ['address', 'lat', 'lng', ]

    def __init__(self, api, addresses=[]):
        """
        Address Instance
        :param api:
        :param addresses:
        :return:
        """
        self.addresses = addresses
        Base.__init__(self, api)

    def get_route_id(self):
        """
        Return Route ID
        :return:
        """
        return self.get_response()['route_id']

    def get_route_destination_id(self):
        """
        Return Destination ID
        :return:
        """
        return self.get_response()['route_destination_id']

    def get_addresses(self):
        """
        Return Addresses
        :return:
        """
        return self.addresses

    def add_address(self, **kwargs):
        """
        Add addresses to optimization
        :param kwargs:
        :return:
        """
        if self.check_required_params(kwargs, self.REQUIRED_FIELDS):
            self.addresses.append(kwargs)
            self.api.optimization.data['addresses'] = self.addresses
        else:
            raise ParamValueException('addresses', 'Params are not complete')

    def batch_fix_geocodes(self, addresses):
        geocoding_error = []
        params = {
            'format': 'json',
        }
        addresses_map = {x['address'].replace('/', ' - '): x for x in addresses}
        data = {'addresses': '||'.join([x['address'] for x in addresses])}
        try:
            json_data = self.get_batch_geocodes(params, data)
        except json.decoder.JSONDecodeError:
            print('Error Geocoding some of the Addresses. Please check the addresses and try again.')
            return addresses, []
        geocoded_addresses = []
        for address in json_data:
            try:
                original_address = addresses_map[address['original']]
                original_address.update({
                    'lat': address['lat'],
                    'lng': address['lng'],
                })
                geocoded_addresses.append(original_address)
            except (IndexError, ValueError):
                geocoding_error.append(address)
        return geocoding_error, addresses

    def get_geocode(self, params):
        """
        Get Geocodes from given address
        :param params:
        :return: response as a object
        """
        self.response = self.api._make_request(SINGLE_GEOCODER,
                                               params,
                                               [],
                                               self.api._request_get)
        return self.response.json()

    def get_batch_geocodes(self, params, data):
        """
        Get Geocodes from given addresses
        :param params:
        :param data:
        :return: response as a object
        """
        self.response = self.api._make_request(BATCH_GEOCODER,
                                               params,
                                               data,
                                               self.api._request_post)
        return self.response.json()

    def fix_geocode(self, address):
        geocoding_error = None
        params = {'format': 'json', 'address': address.get('address')}
        count = 0
        while True:
            try:
                json_data = self.get_geocode(params)
                address.update(json_data)
                return geocoding_error, address
            except (AttributeError, requests.exceptions.ConnectionError):
                count += 1
                if count > 5:
                    geocoding_error = address
                    break
                time.sleep(random.randrange(1, 5) * 0.5)

        return geocoding_error, address

    def request_address(self, params):
        params.update({'api_key': self.api.key})
        return self.api._make_request(ADDRESS_HOST,
                                      params,
                                      None,
                                      self.api._request_get)

    def get_address(self, route_id, route_destination_id):
        params = {'route_id': route_id,
                  'route_destination_id': route_destination_id
                  }
        response = self.request_address(params)
        return response.json()

    def get_address_notes(self, route_id, route_destination_id):
        params = {'route_id': route_id,
                  'route_destination_id': route_destination_id,
                  'notes': True,
                  }
        response = self.request_address(params)
        return response.json()

    def update_address(self, data, route_id, route_destination_id):
        params = {'route_id': route_id,
                  'route_destination_id': route_destination_id
                  }
        params.update({'api_key': self.api.key})
        data = json.dumps(data)
        response = self.api._make_request(ADDRESS_HOST,
                                          params,
                                          data,
                                          self.api._request_put)
        return response.json()

    def delete_address_from_route(self, route_id, route_destination_id):
        params = {'route_id': route_id,
                  'route_destination_id': route_destination_id
                  }
        params.update({'api_key': self.api.key})
        response = self.api._make_request(ADDRESS_HOST,
                                          params,
                                          None,
                                          self.api._request_delete)
        return response.json()

    def add_address_notes(self, note, **kwargs):
        """
        Add Address  Note using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if self.check_required_params(kwargs, ['address_id', 'route_id']):
            data = {'strUpdateType': kwargs.pop('activity_type'),
                    'strNoteContents': note}
            kwargs.update({'api_key': self.params['api_key'], })
            self.response = self.api._request_post(ADD_ROUTE_NOTES_HOST,
                                                   kwargs, data)
            return self.response.json()
        else:
            raise ParamValueException('params', 'Params are not complete')

    def geocode(self, **kwargs):
        """
        Bulk Geocoder using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.
        """
        if 'format' not in kwargs:
            kwargs.update({'format': 'csv'})
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['addresses', ]):
            response = self.api._request_post(BATCH_GEOCODER,
                                              kwargs)
            return response.content

        else:
            raise ParamValueException('params', 'Params are not complete')

# codebeat:enable[ABC]



# -*- coding: utf-8 -*-
# codebeat:disable[TOO_MANY_FUNCTIONS, LOC, ABC, ARITY, TOTAL_LOC]

import requests
import json

try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode
from .activity_feed import ActivityFeed
from .address import Address
from .address_book import AddressBook
from .avoidance_zones import AvoindanceZones
from .exceptions import APIException
from .gps import GPS
from .optimization import Optimization
from .orders import Order
from .rapid_address import RapidAddress
from .route import Route
from .territory import Territory
from .file_uploading import FileUploading
from .members import Members
from .vehicles import Vehicle
from .api_endpoints import API_HOST


HEADERS = {
    'User-Agent': 'python-sdk',
    'Accept-Encoding': 'identity, deflate, compress, gzip',
    'Accept': '*/*',
}


class Route4Me(object):
    """
    Route4Me Python SDK
    """

    def __init__(self,
                 key,
                 headers=HEADERS,
                 redirects=True,
                 verify_ssl=True,
                 proxies={}):
        self.key = key
        self.response = None
        self.activity_feed = ActivityFeed(self)
        self.address = Address(self)
        self.address_book = AddressBook(self)
        self.avoidance_zones = AvoindanceZones(self)
        self.file_uploading = FileUploading(self)
        self.optimization = Optimization(self)
        self.members = Members(self)
        self.vehicles = Vehicle(self)
        self.order = Order(self)
        self.gps = GPS(self)
        self.route = Route(self)
        self.rapid_address = RapidAddress(self)
        self.territory = Territory(self)
        self.headers = headers
        self.redirects = redirects
        self.verify_ssl = verify_ssl
        self.proxies = proxies

    def _make_request(self, url, params, data, request_method):
        """
        Make request to API
        :param url:
        :param params:
        :param data:
        :param request_method:
        :return: response
        :raise: APIException
        """
        params['api_key'] = self.key
        request_params = self._transform_params(params)
        response = request_method(url, request_params, data)
        if not 200 <= response.status_code < 400:
            raise APIException(response.status_code, response.text,
                               response.url)
        return response

    def _transform_params(self, params):
        """
        Convert params dict to url params
        :param params:
        :return:
        """
        return urlencode(params)

    def get(self, request_method):
        """
        Execute optimization
        :param request_method:
        :return: JSON
        """
        params = self.optimization.get_params()
        if not self.redirects:
            params.update({
                'redirect': 0,
            })
        return self._make_request(API_HOST, params,
                                  json.dumps(self.optimization.data),
                                  request_method)

    def _request_post(self, url, request_params, data=None, json=None, files=None):
        """
        POST request
        :param url:
        :param request_params:
        :param data:
        :param files:
        :return:
        """
        return requests.post(url, params=request_params,
                             allow_redirects=self.redirects,
                             proxies=self.proxies, files=files,
                             data=data, headers=self.headers,
                             json=json,
                             verify=self.verify_ssl)

    def _request_get(self, url, request_params, data=None):
        """
        GET request
        :param url:
        :param request_params:
        :param data:
        :return:
        """
        return requests.get(url, params=request_params,
                            allow_redirects=self.redirects,
                            proxies=self.proxies,
                            data=data,
                            headers=self.headers,
                            verify=self.verify_ssl)

    def _request_put(self, url, request_params, json=None, data=None):
        """
        PUT request
        :param url:
        :param request_params:
        :param data:
        :return:
        """
        return requests.request('PUT', url, params=request_params,
                                proxies=self.proxies,
                                data=data,
                                json=json,
                                headers=self.headers,
                                verify=self.verify_ssl)

    def _request_delete(self, url, request_params, data=None):
        """
        DELETE request
        :param url:
        :param request_params:
        :param data:
        :return:
        """
        return requests.request('DELETE', url, params=request_params,
                                data=data,
                                headers=self.headers,
                                verify=self.verify_ssl)

    def run_optimization(self):
        """
        Run optimization and return response as an object.
        :return: response as an object
        """
        self.response = self.get(self._request_post)
        try:
            response = self.response.json()
            return response
        except AttributeError:
            raise
        except ValueError:
            raise
        except Exception:
            raise

    def re_optimization(self, optimization_id, data={}):
        """
        Execute reoptimization
        :param optimization_id:
        :param data:
        :return: response as a object
        """
        self.optimization.optimization_problem_id(optimization_id)
        self.optimization.reoptimize(1)
        data = {'parameters': data}
        self.response = self._make_request(API_HOST,
                                           self.optimization.get_params(),
                                           json.dumps(data),
                                           self._request_put)
        try:
            return self.response.json()
        except ValueError:
            raise
        except Exception:
            raise

    def get_optimization(self, optimization_problem_id):
        """
        Get optimization given optimization_problem_id
        :param optimization_problem_id:
        :return:
        """
        self.optimization.optimization_problem_id(optimization_problem_id)
        self.response = self.get(self._request_get)
        self.parse_response()

    def parse_response(self):
        """
        Parse response and set it to Route4me instance
        :return:
        """
        response = self.response.json()
        if 'addresses' in response:
            self.address.addresses = self.response['addresses']

    def export_result_to_json(self, file_name):
        """
        Export response to JSON File
        :param file_name:
        :return:
        """
        if self.response:
            try:
                f = open(file_name, 'w')
                f.write(json.dumps(self.response.content,
                                   ensure_ascii=False,
                                   sort_keys=True,
                                   indent=4))
                f.close()
            except Exception:
                raise

    def export_request_to_json(self, file_name):
        """
        Export resquest to JSON File
        :param file_name:
        :return:
        """
        if self.optimization.data:
            try:
                f = open(file_name, 'w')
                f.write(json.dumps(self.optimization.data,
                                   ensure_ascii=False,
                                   sort_keys=True,
                                   indent=4))
                f.close()
            except Exception:
                raise


# codebeat:enable[TOO_MANY_FUNCTIONS, LOC, ABC, ARITY, TOTAL_LOC]



# -*- coding: utf-8 -*-
import os

from setuptools import setup
from setuptools import find_packages


from VERSION import PROJECT
from VERSION import COPYRIGHT
from VERSION import AUTHOR
from VERSION import AUTHOR_EMAIL
from VERSION import TITLE
from VERSION import LICENSE
from VERSION import RELEASE_STRING


def read(file_name):
    folder = os.path.dirname(__file__)
    fullname = os.path.join(folder, file_name)
    with open(fullname) as f:
        return f.read()


def rewrite_version():
    with open('VERSION.py', 'r') as inp:
        txt = inp.read()
    outname = os.path.join('route4me', 'version.py')
    with open(outname, 'w') as out:
        out.write(txt)

rewrite_version()


setup(

    name=TITLE,
    url='https://github.com/route4me/route4me-python-sdk',
    bugtrack_url='https://github.com/route4me/route4me-python-sdk/issues',
    copyright=COPYRIGHT,
    author=AUTHOR,
    description=PROJECT,
    version=RELEASE_STRING,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    keywords="rout4me, python, sdk, api",
    packages=find_packages(
        include=['route4me', 'route4me.*'],
        exclude=['*_test*'],
    ),
    zip_safe=True,
    platforms='any',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    classifiers=[
        'Environment :: Other Environment',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    test_suite="tests",
    install_requires=['six', 'requests'],
)



# -*- coding: utf-8 -*-

from .api_endpoints import RAPID_ADDRESS_SERVICE, \
    RAPID_ADDRESS, RAPID_ADDRESS_ZIP
from .base import Base
from .exceptions import ParamValueException


class RapidAddress(Base):
    """
    Rapid Address is a service to get Street Addresses given
    Street name or Zip code
    """

    def __init__(self, api):
        """
        Address Instance
        :param api:
        :return:
        """
        Base.__init__(self, api)

    def get_street_data(self, **kwargs):
        """
        Get Street Data
        :param kwargs:
        :return: API response content
        """
        url = RAPID_ADDRESS
        kwargs.update({'api_key': self.params['api_key'], })
        if 'offset' in kwargs.keys() and 'limit' in kwargs.keys():
            url = '{0}{1}/{2}/'.format(url,
                                       kwargs.pop('offset'),
                                       kwargs.pop('limit'))
        elif 'pk' in kwargs:
            url = '{0}{1}/'.format(url, kwargs.pop('pk'))

        response = self.api._request_get(url, kwargs)
        try:
            return response.json()
        except ValueError:
            return response.content

    def get_street_data_zip(self, **kwargs):
        """
        Get Street Data Given Zipcode
        :param kwargs:
        :return: API response content
        """
        kwargs.update({'api_key': self.params['api_key'], })
        url = RAPID_ADDRESS_ZIP
        if self.check_required_params(kwargs, ['zipcode', 'api_key', ]):
            url = '{0}{1}/'.format(url, kwargs.pop('zipcode'))
            if 'offset' in kwargs.keys() and 'limit' in kwargs.keys():
                url = '{0}{1}/{2}/'.format(url,
                                           kwargs.pop('offset'),
                                           kwargs.pop('limit'))
            response = self.api._request_get(url, kwargs)
            try:
                return response.json()
            except ValueError:
                return response.content
        else:
            raise ParamValueException('params', 'Params are not complete')

    def get_street_data_service(self, **kwargs):
        """
        Get Street Data Given Zipcode and House Number
        :param kwargs:
        :return: API response content
        """
        kwargs.update({'api_key': self.params['api_key'], })
        url = RAPID_ADDRESS_SERVICE
        if self.check_required_params(kwargs, ['zipcode',
                                               'api_key',
                                               'housenumber', ]):
            url = '{0}{1}/{2}/'.format(url,
                                       kwargs.pop('zipcode'),
                                       kwargs.pop('housenumber'))
            if 'offset' in kwargs.keys() and 'limit' in kwargs.keys():
                url = '{0}{1}/{2}/'.format(url,
                                           kwargs.pop('offset'),
                                           kwargs.pop('limit'))
            response = self.api._request_get(url, kwargs)
            try:
                return response.json()
            except ValueError:
                return response.content
        else:
            raise ParamValueException('params', 'Params are not complete')



# -*- coding: utf-8 -*-

from .api_endpoints import FILE_UPLOAD_HOST, \
    FILE_UPLOAD_PREVIEW_HOST, FILE_UPLOAD_GEOCODE_HOST
from .base import Base
from .exceptions import ParamValueException


class FileUploading(Base):
    """
    File Uploading is a service to upload data files
    """

    def __init__(self, api):
        """
        FileUploading Instance
        :param api:
        :return:
        """
        Base.__init__(self, api)

    def get_file_preview(self, **kwargs):
        """
        Get File Preview using GET request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['strUploadID', 'format']):
            self.response = self.api._request_get(FILE_UPLOAD_PREVIEW_HOST,
                                                  kwargs)
            return self.response.content
        else:
            raise ParamValueException('params', 'Params are not complete')

    def upload_file(self, **kwargs):
        """
        Upload File Preview using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['files',
                                               'format', ]):
            self.response = self.api._request_post(FILE_UPLOAD_HOST,
                                                   kwargs,
                                                   files=kwargs.pop('files'))
            return self.response.content
        else:
            raise ParamValueException('params', 'Params are not complete')

    def upload_file_geocode(self, **kwargs):
        """
        Upload File Geocode using POST request
        :return: API response
        :raise: ParamValueException if required params are not present.

        """
        kwargs.update({'api_key': self.params['api_key'], })
        if self.check_required_params(kwargs, ['strUploadID', 'files', ]):
            self.response = self.api._request_post(FILE_UPLOAD_GEOCODE_HOST,
                                                   kwargs,
                                                   files=kwargs.pop('files'))
            return self.response.content
        else:
            raise ParamValueException('params', 'Params are not complete')
